/*
 * Copyright (c) 2007 Henri Sivonen
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS IN THE SOFTWARE.
 */

package fi.iki.hsivonen.bib4ht;

import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLFilterImpl;

import fi.iki.hsivonen.xml.AttributesImpl;
import fi.iki.hsivonen.xml.XhtmlSaxEmitter;

public class TocFilter extends XMLFilterImpl {

    private TocItem root;
    private XhtmlSaxEmitter emitter;

    private final AttributesImpl attrs = new AttributesImpl();
    /**
     * @param root
     */
    public TocFilter(XMLReader parent, TocItem root) {
        super(parent);
        this.root = root;
    }

    /**
     * @see org.xml.sax.helpers.XMLFilterImpl#processingInstruction(java.lang.String, java.lang.String)
     */
    @Override
    public void processingInstruction(String target, String data) throws SAXException {
        if ("toc".equals(target)) {
            emitToc();
        } else {
            super.processingInstruction(target, data);
        }
    }

    /**
     * @see org.xml.sax.helpers.XMLFilterImpl#setContentHandler(org.xml.sax.ContentHandler)
     */
    @Override
    public void setContentHandler(ContentHandler handler) {
        this.emitter = new XhtmlSaxEmitter(handler);
        super.setContentHandler(handler);
    }


    private void emitToc() throws SAXException {
        if (root.hasSubItems()) {
            emitter.startElementWithClass("ul", "toc");
            emitSubItems(root);
            emitter.endElement("ul");
        }
    }

    private void emitSubItems(TocItem parent) throws SAXException {
        for (TocItem item : parent) {
            String klazz = item.getKlazz();
            if (klazz == null) {
                emitter.startElement("li");
            } else {
                emitter.startElementWithClass("li", klazz);
            }

            attrs.clear();
            attrs.addAttribute("href", "#" + item.getId());
            emitter.startElement("a", attrs);
            emitter.characters(item.getContent());
            emitter.endElement("a");
            
            if (item.hasSubItems()) {
                emitter.startElement("ul");
                emitSubItems(item);
                emitter.endElement("ul");
            }
            emitter.endElement("li");
        }
    }
    
    
    
}
