/*
 * Copyright (c) 2007 Henri Sivonen
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS IN THE SOFTWARE.
 */

package fi.iki.hsivonen.bib4ht;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

public class TocGatherer extends DefaultHandler {
    
    private static final Pattern NOTOC = Pattern.compile(".*\\bnotoc\\b.*");

    private int inHeading = 0;

    private TocItem root = new TocItem();

    private TocItem current = root;

    private Locator locator;


    
    /**
     * @see org.xml.sax.helpers.XMLFilterImpl#setDocumentLocator(org.xml.sax.Locator)
     */
    @Override
    public void setDocumentLocator(Locator arg0) {
        this.locator = arg0;
    }
    /**
     * @see org.xml.sax.helpers.DefaultHandler#characters(char[], int, int)
     */
    @Override
    public void characters(char[] arg0, int arg1, int arg2) throws SAXException {
        if (inHeading > 0) {
            current.addContent(arg0, arg1, arg2);
        }
    }

    /**
     * @see org.xml.sax.helpers.DefaultHandler#endElement(java.lang.String,
     *      java.lang.String, java.lang.String)
     */
    @Override
    public void endElement(String uri, String localName, String qName)
            throws SAXException {
        if (inHeading > 0) {
            inHeading--;
        }
    }

    /**
     * @see org.xml.sax.helpers.DefaultHandler#startElement(java.lang.String,
     *      java.lang.String, java.lang.String, org.xml.sax.Attributes)
     */
    @Override
    public void startElement(String uri, String localName, String qName,
            Attributes atts) throws SAXException {
        if (inHeading > 0) {
            inHeading++;
        } else if ("http://www.w3.org/1999/xhtml".equals(uri)
                && ("h2".equals(localName) || "h3".equals(localName)
                        || "h4".equals(localName))) {
            if (atts.getIndex("", "id") < 0) {
                error(new SAXParseException("Heading without id.", locator));
            }
            String klazz = atts.getValue("", "class");
            if (klazz != null) {
                Matcher m = NOTOC.matcher(klazz);
                if (m.matches()) {
                    return;
                }
            }
            current = current.append(new TocItem(klazz,
                    atts.getValue("", "id"), localName));
            inHeading = 1;
        }
    }

    /**
     * Returns the root.
     * 
     * @return the root
     */
    public TocItem getRoot() {
        return root;
    }

}
