/*
 * Copyright (c) 2007 Henri Sivonen
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS IN THE SOFTWARE.
 */

package fi.iki.hsivonen.bib4ht;

import java.util.Iterator;
import java.util.LinkedList;

public class TocItem implements Iterable<TocItem> {
    
    private String klazz;

    private String id;
    
    private int rank;
    
    private StringBuilder content = new StringBuilder();
    
    private TocItem parent = null;
    
    private LinkedList<TocItem> subItems = new LinkedList<TocItem>();
    
    /**
     * @param klazz
     * @param id
     */
    public TocItem(String klazz, String id, String localName) {
        this.klazz = klazz;
        this.id = id;
        this.rank = Integer.parseInt(localName.substring(1));
    }
    
    public TocItem() {
        this.klazz = null;
        this.id = null;
        this.rank = 1;
        this.parent = this;
    }

    public void addContent(char[] buf, int offset, int length) {
        content.append(buf, offset, length);
    }
    
    public char[] getContent() {
        char[] buf = new char[content.length()];
        content.getChars(0, content.length(), buf, 0);
        return buf;
    }

    /**
     * Returns the id.
     * 
     * @return the id
     */
    public String getId() {
        return id;
    }

    /**
     * Returns the klazz.
     * 
     * @return the klazz
     */
    public String getKlazz() {
        return klazz;
    }

    public Iterator<TocItem> iterator() {
        return subItems.iterator();
    }
    
    public TocItem append(TocItem item) {
        if (item.rank == this.rank) {
            this.parent.subItems.addLast(item);
            item.parent = this.parent;
        } else if (item.rank > this.rank) {
            this.subItems.addLast(item);
            item.parent = this;
        } else {
            int delta = this.rank - item.rank;
            TocItem ti = this.parent;
            for (int i = 0; i < delta; i++) {
                ti = ti.parent;
            }
            ti.subItems.addLast(item);
            item.parent = ti;
        }
        return item;
    }

    public boolean hasSubItems() {
        return !subItems.isEmpty();
    }
}
