/*
 * $Id: BibEntry.java,v 1.5 2007/02/26 09:27:27 hsivonen Exp $
 *
 * Copyright (c) 2007 Henri Sivonen and Oskar Ojala.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package net.sourceforge.texlipse.bibparser;

import java.util.HashMap;
import java.util.Map;

/**
 * 
 * 
 * @author Oskar Ojala
 * @author Henri Sivonen
 * 
 */
public class BibEntry implements Comparable {

    private final String key;

    private String type;

    private final Map<String, String> fields = new HashMap<String, String>();

    public BibEntry(String key, String type) {
        this.key = key;
        this.type = type;
    }
    
    public BibEntry(String key) {
        this.key = key;
    }

    public String getKey() {
        return key;
    }

    public String getType() {
        return type;
    }
    
    

    /**
     * @param type the type to set
     */
    protected void setType(String type) {
        this.type = type;
    }

    public void setFieldValue(String name, String value) {
        fields.put(name, value);
    }

    public String getFieldValue(String name) {
        String rv = fields.get(name);
        if (rv == null) {
            return null;
        }
        rv = rv.replace("---", "\u2014");
        rv = rv.replace("--", "\u2013");
        rv = rv.replace("``", "\u201C");
        rv = rv.replace("''", "\u201D");
        rv = rv.replace("`", "\u2018");
        rv = rv.replace("'", "\u2019");
        return rv;
    }

    public boolean hasField(String name) {
        return fields.containsKey(name);
    }

    public String getSymbol() {
        // RFC
        String stdnum = getFieldValue("stdnumber");
        if (stdnum != null && stdnum.startsWith("RFC")) {
            return stdnum.replaceAll(" ", "");
        }
        String symbol = getFieldValue("key");
        if (symbol != null) {
            return symbol;
        }
        return key;
    }

    public int compareTo(Object arg0) {
        String thisKey = this.getSymbol();
        String otherKey = ((BibEntry)arg0).getSymbol();
        return thisKey.compareToIgnoreCase(otherKey);
    }

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object obj) {
        return 0 == compareTo(obj);
    } 

}
